﻿using Hotline.Caching.Interfaces;
using MapsterMapper;
using XF.Domain.Authentications;
using XF.Domain.Repository;
using Hotline.Quality;
using Hotline.Permissions;
using Hotline.Repository.SqlSugar.Extensions;
using Hotline.Settings;
using Hotline.Share.Dtos;
using Hotline.Share.Dtos.File;
using Microsoft.AspNetCore.Mvc;
using XF.Domain.Exceptions;
using Hotline.Share.Dtos.Quality;
using Hotline.Share.Dtos.Order;
using Hotline.Share.Enums.Quality;
using Hotline.Share.Enums.Order;
using XF.Utility.EnumExtensions;
using System.Xml.Linq;
using System.Threading;

namespace Hotline.Api.Controllers
{
	public class QualityController : BaseController
	{
		private readonly ISessionContext _sessionContext;
		private readonly IMapper _mapper;
		private readonly IQualityRepository _qualitey;
		private readonly IRepository<QualityDetail> _qualiteyDetail;
		private readonly IRepository<QualityItem> _qualiteyItem;
		private readonly IRepository<QualityTemplate> _qualityTemplate;
		private readonly IRepository<QualityTemplateDetail> _qualiteyTemplateDetail;
		private readonly IRepository<QualityProhibited> _qualiteyProhibited;
		private readonly ISystemDomainService _systemDomainService;

		public QualityController(
			ISessionContext sessionContext,
			IMapper mapper,
			IQualityRepository qualitey,
			IRepository<Quality.QualityDetail> qualiteyDetail,
			IRepository<QualityItem> qualiteyItem,
			IRepository<QualityTemplate> qualityTemplate,
			IRepository<QualityTemplateDetail> qualiteyTemplateDetail,
			IRepository<QualityProhibited> qualiteyProhibited,
			ISystemDomainService systemDomainService
		)
		{
			_sessionContext = sessionContext;
			_mapper = mapper;
			_qualitey = qualitey;
			_qualiteyDetail = qualiteyDetail;
			_qualiteyItem = qualiteyItem;
			_qualityTemplate = qualityTemplate;
			_qualiteyTemplateDetail = qualiteyTemplateDetail;
			_qualiteyProhibited = qualiteyProhibited;
			_systemDomainService = systemDomainService;
		}
		#region 质检管理
		/// <summary>
		/// 删除质检
		/// </summary>
		/// <param name="dto"></param>
		/// <returns></returns>
		[Permission(EPermission.DeleteQuality)]
		[HttpDelete]
		public async Task Delete([FromBody] DeleteQualityDto dto)
		{
			foreach (var Id in dto.Ids)
			{
				await _qualitey.RemoveAsync(x => x.Id == Id);
				List<QualityDetail> details = await _qualiteyDetail.Queryable().Where(x => x.QualityId == Id).ToListAsync();
				await _qualiteyDetail.RemoveRangeAsync(details, HttpContext.RequestAborted);
			}
		}

		/// <summary>
		/// 更新质检
		/// </summary>
		/// <param name="dto"></param>
		/// <returns></returns>
		[Permission(EPermission.UpdateQuality)]
		[HttpPut]
		public async Task Update([FromBody] UpdateQualityDto dto)
		{
			await _qualitey.UpdateQualityAsync(dto, HttpContext.RequestAborted);
		}

		/// <summary>
		/// 获取质检列表
		/// </summary>
		/// <param name="dto"></param>
		/// <returns></returns>
		[Permission(EPermission.QualityList)]
		[HttpGet("list")]
		public async Task<PagedDto<QualityDto>> List([FromQuery] QualityListDto dto)
		{
			var (total, items) = await _qualitey.Queryable()
				.Includes(x => x.Order)
				.Includes(x => x.QualityDetails)
				.WhereIF(!string.IsNullOrEmpty(dto.Keyword), d => d.Order.Title.Contains(dto.Keyword!) || d.Order.No.Contains(dto.Keyword!))
				.WhereIF(dto.State.HasValue, d => d.State == dto.State)
				.WhereIF(dto.Source.HasValue, d => d.Source == dto.Source)
				.WhereIF(dto.CreationTimeStart.HasValue, d => d.CreationTime >= dto.CreationTimeStart)
				.WhereIF(dto.CreationTimeEnd.HasValue, d => d.CreationTime <= dto.CreationTimeEnd)
				.OrderByDescending(x => x.CreationTime)
				.ToPagedListAsync(dto.PageIndex, dto.PageSize, HttpContext.RequestAborted);
			return new PagedDto<QualityDto>(total, _mapper.Map<IReadOnlyList<QualityDto>>(items));
		}

		/// <summary>
		/// 获取质检实体
		/// </summary>
		/// <param name="id"></param>
		/// <returns></returns>
		[Permission(EPermission.QualityEntity)]
		[HttpGet("{id}")]
		public async Task<Quality.Quality> Entity(string id)
		{
			return await _qualitey.Queryable()
				.Includes(x => x.QualityDetails)
				.FirstAsync(x => x.Id == id);
		}
		#endregion
		#region 质检项管理 
		/// <summary>
		/// 新增项目
		/// </summary>
		/// <param name="dtos"></param>
		/// <returns></returns>
		[Permission(EPermission.AddQualityItem)]
		[HttpPost("item")]
		public async Task Add([FromBody] QualityItemAddDto dto)
		{
			var model = _mapper.Map<QualityItem>(dto);
			await _qualiteyItem.AddAsync(model, HttpContext.RequestAborted);
		}

		/// <summary>
		/// 删除项目
		/// </summary>
		/// <param name="dto"></param>
		/// <returns></returns>
		[Permission(EPermission.DeleteQualityItem)]
		[HttpDelete("item")]
		public async Task Delete([FromBody] QualityItemDeleteDto dto)
		{
			await _qualiteyItem.RemoveAsync(x => x.Id == dto.Id);
		}

		/// <summary>
		/// 更新项目
		/// </summary>
		/// <param name="dto"></param>
		/// <returns></returns>
		[Permission(EPermission.UpdateQualityItem)]
		[HttpPut("item")]
		public async Task Update([FromBody] QualityItemUpdateDto dto)
		{
			var item = await _qualiteyItem.GetAsync(dto.Id, HttpContext.RequestAborted);
			if (item is null)
				throw UserFriendlyException.SameMessage("无效质检项目");
			_mapper.Map(dto, item);
			await _qualiteyItem.UpdateAsync(item, HttpContext.RequestAborted);
		}

		/// <summary>
		/// 获取项目列表
		/// </summary>
		/// <param name="dto"></param>
		/// <returns></returns>
		[Permission(EPermission.QualityItemList)]
		[HttpGet("item/list")]
		public async Task<PagedDto<QualityItemDto>> List([FromQuery] QualityItemListDto dto)
		{
			var (total, items) = await _qualiteyItem.Queryable()
				.WhereIF(!string.IsNullOrEmpty(dto.Name), d => d.Name.Contains(dto.Name!))
				.WhereIF(!string.IsNullOrEmpty(dto.GroupingName), d => d.GroupingName.Contains(dto.GroupingName!))
				.OrderByDescending(x => x.CreationTime)
				.ToPagedListAsync(dto.PageIndex, dto.PageSize, HttpContext.RequestAborted);
			return new PagedDto<QualityItemDto>(total, _mapper.Map<IReadOnlyList<QualityItemDto>>(items));
		}

		/// <summary>
		/// 获取项目实体
		/// </summary>
		/// <param name="id"></param>
		/// <returns></returns>
		[Permission(EPermission.QualityItemEntity)]
		[HttpGet("item/{id}")]
		public async Task<QualityItem> ItemEntity(string id)
		{
			return await _qualiteyItem.Queryable()
				.FirstAsync(x => x.Id == id);
		}

		#endregion
		#region 质检模版管理
		/// <summary>
		/// 新增模版
		/// </summary>
		/// <param name="dtos"></param>
		/// <returns></returns>
		[Permission(EPermission.AddQualityTemplate)]
		[HttpPost("template")]
		public async Task Add([FromBody] TemplateAddDto dto)
		{
			var model = _mapper.Map<QualityTemplate>(dto);
			var id = await _qualityTemplate.AddAsync(model, HttpContext.RequestAborted);
			if (string.IsNullOrEmpty(id))
			{
				foreach (var item in dto.TemplateDetails)
				{
					item.TemplateId = id;
				}
				List<QualityTemplateDetail> details = _mapper.Map<List<QualityTemplateDetail>>(dto.TemplateDetails);
				await _qualiteyTemplateDetail.AddRangeAsync(details, HttpContext.RequestAborted);
			}
		}

		/// <summary>
		/// 删除模版
		/// </summary>
		/// <param name="dto"></param>
		/// <returns></returns>
		[Permission(EPermission.DeleteQualityTemplate)]
		[HttpDelete("template")]
		public async Task Delete([FromBody] TemplateDeleteDto dto)
		{
			await _qualityTemplate.RemoveAsync(x => x.Id == dto.Id);
			List<QualityTemplateDetail> details = await _qualiteyTemplateDetail.Queryable().Where(x => x.TemplateId == dto.Id).ToListAsync();
			await _qualiteyTemplateDetail.RemoveRangeAsync(details, HttpContext.RequestAborted);
		}

		/// <summary>
		/// 更新模版
		/// </summary>
		/// <param name="dto"></param>
		/// <returns></returns>
		[Permission(EPermission.UpdateQualityTemplate)]
		[HttpPut("template")]
		public async Task Update([FromBody] TemplateUpdateDto dto)
		{
			var template = await _qualityTemplate.GetAsync(dto.Id, HttpContext.RequestAborted);
			if (template is null)
				throw UserFriendlyException.SameMessage("无效质检模版");
			_mapper.Map(dto, template);
			await _qualityTemplate.UpdateAsync(template, HttpContext.RequestAborted);

			List<QualityTemplateDetail> details = await _qualiteyTemplateDetail.Queryable().Where(x => x.TemplateId == dto.Id).ToListAsync();
			await _qualiteyTemplateDetail.RemoveRangeAsync(details, HttpContext.RequestAborted);
			foreach (var item in dto.TemplateDetails)
			{
				item.TemplateId = dto.Id;
			}
			List<QualityTemplateDetail> newDetails = _mapper.Map<List<QualityTemplateDetail>>(dto.TemplateDetails);
			await _qualiteyTemplateDetail.AddRangeAsync(newDetails, HttpContext.RequestAborted);
		}

		/// <summary>
		/// 获取模版列表
		/// </summary>
		/// <param name="dto"></param>
		/// <returns></returns>
		[Permission(EPermission.QualityTemplateList)]
		[HttpGet("template/list")]
		public async Task<PagedDto<QualityTemplateDto>> List([FromQuery] TemplateListDto dto)
		{
			var (total, items) = await _qualityTemplate.Queryable()
				.Includes(x => x.QualityTemplates)
				.WhereIF(!string.IsNullOrEmpty(dto.Name), d => d.Name.Contains(dto.Name!))
				.WhereIF(dto.Grouping.HasValue, d => d.Grouping == dto.Grouping)
				.OrderByDescending(x => x.CreationTime)
				.ToPagedListAsync(dto.PageIndex, dto.PageSize, HttpContext.RequestAborted);
			return new PagedDto<QualityTemplateDto>(total, _mapper.Map<IReadOnlyList<QualityTemplateDto>>(items));
		}

		/// <summary>
		/// 获取模版实体
		/// </summary>
		/// <param name="id"></param>
		/// <returns></returns>
		[Permission(EPermission.QualityTemplateEntity)]
		[HttpGet("template/{id}")]
		public async Task<QualityTemplate> TemplateEntity(string id)
		{
			return await _qualityTemplate.Queryable()
				.Includes(x => x.QualityTemplates)
				.FirstAsync(x => x.Id == id);
		}
		/// <summary>
		/// 启禁用
		/// </summary>
		/// <param name="dto"></param>
		/// <returns></returns>
		[Permission(EPermission.EnableQualityTemplate)]
		[HttpPut("template/enable")]
		public async Task Enable([FromBody] TemplateUpdateDto dto)
		{
			var template = await _qualityTemplate.GetAsync(dto.Id, HttpContext.RequestAborted);
			if (template is null)
				throw UserFriendlyException.SameMessage("无效质检模版");
			_mapper.Map(dto, template);
			await _qualityTemplate.UpdateAsync(template, HttpContext.RequestAborted);
		}
		#endregion
		#region 质检词库管理

		/// <summary>
		/// 新增违禁词
		/// </summary>
		/// <param name="dtos"></param>
		/// <returns></returns>
		[Permission(EPermission.AddQualityProhibited)]
		[HttpPost("prohibited")]
		public async Task Add([FromBody] ProhibitedAddDto dto)
		{
			var model = _mapper.Map<QualityProhibited>(dto);
			 await _qualiteyProhibited.AddAsync(model, HttpContext.RequestAborted);
		}

		/// <summary>
		/// 删除违禁词
		/// </summary>
		/// <param name="dto"></param>
		/// <returns></returns>
		[Permission(EPermission.DeleteQualityProhibited)]
		[HttpDelete("prohibited")]
		public async Task Delete([FromBody] ProhibitedDeleteDto dto)
		{
			await _qualiteyProhibited.RemoveAsync(x => x.Id == dto.Id);
		}

		/// <summary>
		/// 更新违禁词
		/// </summary>
		/// <param name="dto"></param>
		/// <returns></returns>
		[Permission(EPermission.UpdateQualityProhibited)]
		[HttpPut("prohibited")]
		public async Task Update([FromBody] ProhibitedUpdateDto dto)
		{
			var template = await _qualiteyProhibited.GetAsync(dto.Id, HttpContext.RequestAborted);
			if (template is null)
				throw UserFriendlyException.SameMessage("无效质检模版");
			_mapper.Map(dto, template);
			await _qualiteyProhibited.UpdateAsync(template, HttpContext.RequestAborted);
		}

		/// <summary>
		/// 获取违禁词列表
		/// </summary>
		/// <param name="dto"></param>
		/// <returns></returns>
		[Permission(EPermission.QualityProhibitedList)]
		[HttpGet("prohibited/list")]
		public async Task<PagedDto<QualityProhibitedDto>> List([FromQuery] ProhibitedListDto dto)
		{
			var (total, items) = await _qualiteyProhibited.Queryable()
				.WhereIF(!string.IsNullOrEmpty(dto.Name), d => d.Name.Contains(dto.Name!))
				.WhereIF(!string.IsNullOrEmpty(dto.Classify), d => d.Classify.Contains(dto.Classify!))
				.WhereIF(!string.IsNullOrEmpty(dto.Type), d => d.Type.Contains(dto.Type!))
				.OrderByDescending(x => x.CreationTime)
				.ToPagedListAsync(dto.PageIndex, dto.PageSize, HttpContext.RequestAborted);
			return new PagedDto<QualityProhibitedDto>(total, _mapper.Map<IReadOnlyList<QualityProhibitedDto>>(items));
		}

		/// <summary>
		/// 获取违禁词实体
		/// </summary>
		/// <param name="id"></param>
		/// <returns></returns>
		[Permission(EPermission.QualityTemplateEntity)]
		[HttpGet("prohibited/{id}")]
		public async Task<QualityProhibited> ProhibitedEntity(string id)
		{
			return await _qualiteyProhibited.Queryable()
				.FirstAsync(x => x.Id == id);
		}
		#endregion

		/// <summary>
		/// 获取质检基本信息
		/// </summary>
		/// <returns></returns>
		[HttpGet("base")]
		public async Task<object> Base()
		{
			var rsp = new
			{
				QualityState = EnumExts.GetDescriptions<EQualityState>()
			};
			return Task.FromResult<object>(rsp);
		}

		/// <summary>
		/// 获取质检项目基本信息
		/// </summary>
		/// <returns></returns>
		[HttpGet("item_base")]
		public async Task<object> ItemBase()
		{
			var rsp = new
			{
				QualityItem = EnumExts.GetDescriptions<EQualityItem>(),
				QualityItemGrouping = await _qualiteyItem.Queryable().Where(x => x.Type == EQualityItem.Grouping).ToListAsync()
			};
			return Task.FromResult<object>(rsp);
		}

		/// <summary>
		/// 获取质检模版基本信息
		/// </summary>
		/// <returns></returns>
		[HttpGet("template_base")]
		public async Task<object> TemplateBase()
		{
			var rsp = new
			{
				TemplateGrouping = EnumExts.GetDescriptions<ETemplateGrouping>()
			};
			return Task.FromResult<object>(rsp);
		}

		/// <summary>
		/// 获取质检违禁词基本信息
		/// </summary>
		/// <returns></returns>
		[HttpGet("prohibited_base")]
		public async Task<object> ProhibitedBase()
		{
			var rsp = new
			{
				ProhibitedClassify = await _systemDomainService.GetSysDicDataByCodeAsync(SysDicTypeConsts.ProhibitedClassify),
				ProhibitedType = await _systemDomainService.GetSysDicDataByCodeAsync(SysDicTypeConsts.ProhibitedType),
			};
			return Task.FromResult<object>(rsp);
		}
	}
}
