﻿using Hotline.FlowEngine.Definitions;
using Hotline.Orders;
using Hotline.Share.Dtos;
using Hotline.Share.Enums.FlowEngine;
using Hotline.Share.Enums.Order;
using Hotline.Share.Enums.Settings;
using SqlSugar;
using XF.Domain.Entities;
using XF.Domain.Exceptions;
using XF.Domain.Extensions;
using XF.Domain.Repository;

namespace Hotline.FlowEngine.Workflows;

public class Workflow : CreationEntity
{
    public string DefinitionId { get; set; }

    #region 业务模块

    [SugarColumn(IsNullable = true)]
    public string? ModuleId { get; set; }

    [SugarColumn(IsNullable = true)]
    public string? ModuleName { get; set; }

    [SugarColumn(IsNullable = true)]
    public string? ModuleCode { get; set; }

    #endregion

    [SugarColumn(Length = 2000)]
    public string Title { get; set; }

    /// <summary>
    /// 到期时间(期满时间)
    /// </summary>
    public DateTime ExpiredTime { get; set; }

    /// <summary>
    /// 办理时间限制(如：24小时、7个工作日)
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? TimeLimit { get; set; }

    public int? TimeLimitCount { get; set; }

    public ETimeType? TimeLimitUnit { get; set; } = ETimeType.WorkDay;

    /// <summary>
    /// 流程流转到end节点的时间（配置模板的end节点前一个节点办理完成时间）
    /// </summary>
    public DateTime? CompleteTime { get; set; }

    /// <summary>
    /// 流程状态
    /// </summary>
    public EWorkflowStatus Status { get; set; }

    #region 实际办理信息（节点，部门，意见）

    /// <summary>
    /// 实际办理节点code（会签状态此字段保存最外层会签发起节点code）
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? ActualHandleStepCode { get; set; }

    /// <summary>
    /// 实际办理节点名称（会签状态此字段保存最外层会签发起节点名称）
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? ActualHandleStepName { get; set; }

    /// <summary>
    /// 实际办理节点id
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? ActualHandleStepId { get; set; }

    /// <summary>
    /// 到达实际办理节点时间（stepBox创建时间）
    /// </summary>
    public DateTime? ActualHandleStepCreateTime { get; set; }

    /// <summary>
    /// 实际办理节点签收时间
    /// </summary>
    public DateTime? ActualHandleStepAcceptTime { get; set; }

    /// <summary>
    /// 实际办理时间
    /// </summary>
    public DateTime? ActualHandleTime { get; set; }

    /// <summary>
    /// 实际办理人id
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? ActualHandlerId { get; set; }

    /// <summary>
    /// 实际办理人名称
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? ActualHandlerName { get; set; }

    /// <summary>
    /// 实际办理部门名称
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? ActualHandleOrgName { get; set; }

    /// <summary>
    /// 实际办理部门编码
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? ActualHandleOrgCode { get; set; }

    /// <summary>
    /// 实际办理部门行政区划编码
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? ActualHandleOrgAreaCode { get; set; }

    /// <summary>
    /// 实际办理部门行政区划名称
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? ActualHandleOrgAreaName { get; set; }

    /// <summary>
    /// 实际办理意见（办理中...or 最终办理意见）
    /// </summary>
    [SugarColumn(Length = 2000)]
    public string ActualOpinion { get; set; } = "办理中...";

    /// <summary>
    /// 真实办理人姓名（手动填写）
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? RealHandlerName { get; set; }

    /// <summary>
    /// 真实办理人电话（手动填写）
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? RealHandlerPhone { get; set; }

    /// <summary>
    /// 沟通方式（手动填写）
    /// </summary>
    public ERealCommunicationMode? RealCommunicationMode { get; set; }

    /// <summary>
    /// 沟通时间（手动填写）
    /// </summary>
    public DateTime? RealCommunicationTime { get; set; }

    /// <summary>
    /// 沟通地点（手动填写）
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? RealCommunicationAddress { get; set; }

    #endregion

    #region 当前办理节点信息

    /// <summary>
    /// 当前办理节点code（非会签：当前被指派节点，会签：会签发起节点）
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? CurrentStepCode { get; set; }

    /// <summary>
    /// 当前节点名称
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? CurrentStepName { get; set; }

    /// <summary>
    /// 当前节点id
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? CurrentStepId { get; set; }

    /// <summary>
    /// 到达当前节点时间（stepBox创建时间）
    /// </summary>
    public DateTime? CurrentStepCreateTime { get; set; }

    #endregion

    /// <summary>
    /// 会签类型
    /// </summary>
    public ECounterSignType? CounterSignType { get; set; }

    /// <summary>
    /// 会签办理节点code，嵌套会签为最外层会签办理节点code（不处于会签状态则无值）
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? TopCountersignStepCode { get; set; }

    /// <summary>
    /// 处理方式（直办、交办）
    /// </summary>
    [SugarColumn(DefaultValue = "10")]
    public EProcessType ProcessType { get; set; } = EProcessType.Zhiban;

    /// <summary>
    /// 交办时间(中心交部门办理时间)
    /// </summary>
    public DateTime? CenterToOrgTime { get; set; }

    /// <summary>
    /// 办结时长（分钟）
    /// 办结时间-交办时间
    /// </summary>
    [SugarColumn(DefaultValue = "0")]
    public double HandleDuration { get; set; }

    /// <summary>
    /// 办结工作日时长
    /// </summary>
    [SugarColumn(DefaultValue = "0")]
    public double HandleDurationWorkday { get; set; }

    /// <summary>
    /// 全流程时长（分钟）
    /// 归档时间-创建时间
    /// </summary>
    [SugarColumn(DefaultValue = "0")]
    public double AllDuration { get; set; }

    /// <summary>
    /// 外部业务唯一标识
    /// </summary>
    public string ExternalId { get; set; }

    #region 办理对象

    /// <summary>
    /// 办理人id
    /// </summary>
    [SugarColumn(ColumnDataType = "json", IsJson = true)]
    public List<HandlerGroupItem> HandlerUsers { get; set; } = new();

    /// <summary>
    /// 办理部门code
    /// </summary>
    [SugarColumn(ColumnDataType = "json", IsJson = true)]
    public List<HandlerGroupItem> HandlerOrgs { get; set; } = new();

    #endregion

    #region 查看对象

    /// <summary>
    /// 指派部门code（所有流经部门留痕）
    /// </summary>
    [SugarColumn(ColumnDataType = "json", IsJson = true)]
    public List<string> AssignOrgCodes { get; set; } = new();

    /// <summary>
    /// 指派办理人Id（所有流经办理人留痕）
    /// </summary>
    [SugarColumn(ColumnDataType = "json", IsJson = true)]
    public List<string> AssignUserIds { get; set; } = new();

    #endregion

    #region 一级部门

    /// <summary>
    /// 一级部门code
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? OrgLevelOneCode { get; set; }

    /// <summary>
    /// 一级部门名称
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? OrgLevelOneName { get; set; }

    #endregion

    #region 受理人(开启流程的话务员)

    /// <summary>
    /// 受理人id
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? AcceptorId { get; set; }

    /// <summary>
    /// 受理人名称
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? AcceptorName { get; set; }

    /// <summary>
    /// 受理人工号
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? AcceptorStaffNo { get; set; }

    /// <summary>
    /// 受理人部门编码
    /// </summary>
    public string? AcceptorOrgCode { get; set; }

    /// <summary>
    /// 受理人部门名称
    /// </summary>
    public string? AcceptorOrgName { get; set; }

    #endregion

    [Navigate(NavigateType.OneToOne, nameof(DefinitionId))]
    public Definition Definition { get; set; }

    /// <summary>
    /// 补充信息
    /// </summary>
    [Navigate(NavigateType.OneToMany, nameof(WorkflowSupplement.WorkflowId))]
    public List<WorkflowSupplement> Supplements { get; set; }

    ///// <summary>
    ///// 接办信息
    ///// </summary>
    //[Navigate(NavigateType.OneToMany, nameof(WorkflowAssign.WorkflowId))]
    //public List<WorkflowAssign> Assigns { get; set; }

    /// <summary>
    /// 会签
    /// </summary>
    [Navigate(NavigateType.OneToMany, nameof(WorkflowCountersign.WorkflowId))]
    public List<WorkflowCountersign> Countersigns { get; set; }

    /// <summary>
    /// 主节点，依据流转进度动态生成或删除
    /// </summary>
    [SugarColumn(IsIgnore = true)]
    public List<WorkflowStep> StepBoxes { get; set; }

    [SugarColumn(IsIgnore = true)]
    public List<WorkflowTrace> Traces { get; set; } = new();

    #region Method

    /// <summary>
    /// 流程结束
    /// </summary>
    public void Complete()
    {
        Status = EWorkflowStatus.Completed;
        CompleteTime = DateTime.Now;
        SetAllDuration();

        UpdateActualOption();
    }

    public void Terminate(string opinion)
    {
        Status = EWorkflowStatus.Terminated;
        CompleteTime = DateTime.Now;
        ActualOpinion = opinion;
    }

    /// <summary>
    /// 更新实际办理节点、部门数据
    /// </summary>
    private void ActualHandle(
        WorkflowStep currentStep,
        string? actualHandlerId,
        string? actualHandlerName,
        string? actualHandleOrgCode,
        string? actualHandleOrgName,
        string? actualHandleOrgAreaCode,
        string? actualHandleOrgAreaName)
    {
        ActualHandleStepCode = currentStep.Code;
        ActualHandleStepName = currentStep.Name;
        ActualHandleStepId = currentStep.Id;
        ActualHandleStepCreateTime = currentStep.CreationTime;
        ActualHandleTime = DateTime.Now;
        ActualHandlerId = actualHandlerId;
        ActualHandlerName = actualHandlerName;
        ActualHandleOrgCode = actualHandleOrgCode;
        ActualHandleOrgName = actualHandleOrgName;
        ActualHandleOrgAreaCode = actualHandleOrgAreaCode;
        ActualHandleOrgAreaName = actualHandleOrgAreaName;
    }

    /// <summary>
    /// 归档时为最终办理意见赋值
    /// </summary>
    public void UpdateActualOption()
    {
        var stepBox = StepBoxes.FirstOrDefault(d => d.Code == ActualHandleStepCode);
        if (stepBox is null)
            throw new UserFriendlyException($"未查询到对应实际办理节点，workflowId: {Id}");
        var step = stepBox.Steps.FirstOrDefault(d => d.Id == ActualHandleStepId);
        if (step is null)
            throw new UserFriendlyException($"未查询到对应实际办理子节点，workflowId: {Id}");
        ActualOpinion = step.Opinion;
        ActualHandleStepAcceptTime = step.AcceptTime;
    }

    /// <summary>
    /// 更新workflow中实际办理节点数据（节点，时间，部门）
    /// </summary>
    public void UpdateWorkflowActualHandleInfo(WorkflowStep currentStepBox, WorkflowStep currentStep,
        string actualHandlerId, string? actualHandlerName,
        string actualHandleOrgCode, string? actualHandleOrgName,
        string? actualHandleOrgAreaCode, string? actualHandleOrgAreaName)
    {
        //a.会签中分为1.之前无会签，当前节点发起会签，记录当前节点 2. 之前有会签，不更新  b.非会签中，记录当前节点
        if (IsInCountersign() && !currentStep.HasStartedCountersign) return;

        if (currentStepBox.StepType is EStepType.CountersignEnd or EStepType.End) return;

        ActualHandle(currentStep,
            actualHandlerId, actualHandlerName,
            actualHandleOrgCode, actualHandleOrgName,
            actualHandleOrgAreaCode, actualHandleOrgAreaName);

        SetHandleDuration();
    }

    /// <summary>
    /// 更新当前办理节点信息
    /// </summary>
    public void UpdateWorkflowCurrentStepInfo(bool isStartCountersign, WorkflowStep? currentStep = null, WorkflowStep? nextStep = null)
    {
        //非会签：指当前被指派节点，会签：会签发起节点
        if (IsInCountersign()) return;

        if (isStartCountersign && currentStep is null)
            throw new UserFriendlyException("开启会签时，会签发起节点不能为空");

        if (!isStartCountersign && nextStep is null)
            throw new UserFriendlyException("未开启会签时，被指派节点不能为空");

        CurrentHandle(isStartCountersign ? currentStep : nextStep);
    }

    public void CurrentHandle(WorkflowStep step)
    {
        CurrentStepCode = step.Code;
        CurrentStepName = step.Name;
        CurrentStepId = step.Id;
        CurrentStepCreateTime = step.CreationTime;
    }

    /// <summary>
    /// 检查会签是否结束
    /// </summary>
    public bool CheckIfCountersignOver()
    {
        return Countersigns.All(d => d.IsCompleted());
    }

    /// <summary>
    /// 流程是否处于会签中
    /// </summary>
    /// <returns></returns>
    public bool IsInCountersign() => !string.IsNullOrEmpty(TopCountersignStepCode);

    /// <summary>
    /// 开始会签
    /// </summary>
    public void StartCountersign(string currentCountersignCode, ECounterSignType counterSignType)
    {
        TopCountersignStepCode = currentCountersignCode;
        CounterSignType = counterSignType;
    }

    /// <summary>
    /// 结束流程会签状态
    /// </summary>
    public void EndCountersign() => TopCountersignStepCode = null;

    /// <summary>
    /// 重置最终办理意见
    /// </summary>
    public void ResetOption() => ActualOpinion = "办理中...";

    /// <summary>
    /// 重新设置办理人（删除当前待办人/部门），撤回/跳转场景，因当前办理人不是流程指定办理人
    /// </summary>
    /// <param name="assignType"></param>
    /// <param name="handlerObjects"></param>
    public void ResetHandlers(EFlowAssignType assignType, List<HandlerGroupItem> handlerObjects)
    {
        HandlerOrgs = new List<HandlerGroupItem>();
        HandlerUsers = new List<HandlerGroupItem>();

        SetHandlers(assignType, handlerObjects);
    }

    /// <summary>
    /// 更新当前办理人(待办人或部门)，正常流转场景
    /// </summary>
    public void UpdateHandlers(string handlerId, string handlerOrg, EFlowAssignType assignType, List<HandlerGroupItem> handlerObjects)
    {
        RemoveCurrentHandleGroup(handlerId, handlerOrg);

        SetHandlers(assignType, handlerObjects);
    }

    private void SetHandlers(EFlowAssignType assignType, List<HandlerGroupItem> handlerObjects)
    {
        switch (assignType)
        {
            case EFlowAssignType.Org:
                HandlerOrgs.AddRange(handlerObjects);
                HandlerOrgs = HandlerOrgs.Distinct().ToList();
                break;
            case EFlowAssignType.User:
                HandlerUsers.AddRange(handlerObjects);
                HandlerUsers = HandlerUsers.Distinct().ToList();
                break;
            default:
                throw new ArgumentOutOfRangeException(nameof(assignType), assignType, null);
        }
    }

    /// <summary>
    /// 更新当前办理人（逐级退回场景）
    /// </summary>
    /// <param name="handlerId"></param>
    /// <param name="handlerOrg"></param>
    /// <param name="prevStep"></param>
    public void UpdatePreviousHandlers(string handlerId, string handlerOrg, WorkflowStep prevStep)
    {
        RemoveCurrentHandleGroup(handlerId, handlerOrg);

        var groupId = Guid.NewGuid().ToString();
        var handlerObjects = prevStep.Handlers.Select(d => new HandlerGroupItem
        {
            GroupId = groupId,
            Id = d.Id,
            Name = d.Name
        });
        switch (prevStep.HandlerType)
        {
            case EHandlerType.OrgLevel:
            case EHandlerType.OrgType:
            case EHandlerType.AssignOrg:
                HandlerOrgs.AddRange(handlerObjects);
                HandlerOrgs = HandlerOrgs.Distinct().ToList();
                break;
            case EHandlerType.Role:
            case EHandlerType.AssignUser:
                HandlerUsers.AddRange(handlerObjects);
                HandlerUsers = HandlerUsers.Distinct().ToList();
                break;
            default:
                throw new ArgumentOutOfRangeException();
        }
    }

    /// <summary>
    /// 当前用户是否可以办理该流程
    /// </summary>
    /// <returns></returns>
    public bool CanHandle(string userId, string orgCode) =>
        Status is EWorkflowStatus.Runnable &&
        (HandlerUsers.Any(d => d.Id == userId) || HandlerOrgs.Any(d => d.Id == orgCode));

    private void RemoveCurrentHandleGroup(string handlerId, string handlerOrg)
    {
        if (string.IsNullOrEmpty(handlerId) && string.IsNullOrEmpty(handlerOrg))
            throw new UserFriendlyException($"{nameof(RemoveCurrentHandleGroup)}, 办理对象参数为空");
        var orgItem = HandlerOrgs.FirstOrDefault(d => d.Id == handlerOrg);
        if (orgItem != null)
        {
            HandlerOrgs.RemoveAll(d => d.GroupId == orgItem.GroupId);
        }

        var userItem = HandlerUsers.FirstOrDefault(d => d.Id == handlerId);
        if (userItem != null)
        {
            HandlerUsers.RemoveAll(d => d.GroupId == userItem.GroupId);
        }
    }

    public void Assign(EFlowAssignType type, string handler)
    {
        handler = handler.ToLower();
        switch (type)
        {
            case EFlowAssignType.Org:
                var orgCodes = handler.GetHigherOrgCodes(true).ToList();
                AssignOrgCodes.AddRange(orgCodes);
                AssignOrgCodes = AssignOrgCodes.Distinct().ToList();
                break;
            case EFlowAssignType.User:
                if (!AssignUserIds.Exists(d => d == handler))
                    AssignUserIds.Add(handler);
                break;
            default:
                throw new ArgumentOutOfRangeException(nameof(type), type, null);
        }
    }

    public void Assign(EFlowAssignType type, IEnumerable<string> handlers)
    {
        handlers = handlers.Select(d => d.ToLower());
        switch (type)
        {
            case EFlowAssignType.Org:
                var orgCodes = handlers.SelectMany(d => d.GetHigherOrgCodes(true)).ToList();
                AssignOrgCodes.AddRange(orgCodes);
                AssignOrgCodes = AssignOrgCodes.Distinct().ToList();
                break;
            case EFlowAssignType.User:
                AssignUserIds.AddRange(handlers);
                AssignUserIds = AssignUserIds.Distinct().ToList();
                break;
            default:
                throw new ArgumentOutOfRangeException(nameof(type), type, null);
        }
    }

    /// <summary>
    /// 更新受理人信息
    /// </summary>
    public void UpdateAcceptor(string userId, string? userName, string? staffNo, string orgCode, string? orgName)
    {
        AcceptorId = userId;
        AcceptorName = userName;
        AcceptorStaffNo = staffNo;
        AcceptorOrgCode = orgCode;
        AcceptorOrgName = orgName;
    }

    /// <summary>
    /// 更新一级部门信息
    /// </summary>
    public void UpdateOrgLevelOne(string orgCode, string orgName)
    {
        OrgLevelOneCode = orgCode;
        OrgLevelOneName = orgName;
    }

    public void CenterToOrg(DateTime expiredTime)
    {
        ProcessType = EProcessType.Jiaoban;
        ExpiredTime = expiredTime;
        CenterToOrgTime = DateTime.Now;
    }

    public void OrgToCenter(DateTime expiredTime)
    {
        ProcessType = EProcessType.Zhiban;
        ExpiredTime = expiredTime;
        CenterToOrgTime = null;
    }

    public void Redo()
    {
        ResetOption();
        Status = EWorkflowStatus.Runnable;

    }

    public void SetHandleDuration()
    {
        if (!ActualHandleTime.HasValue)
            throw new ArgumentNullException($"{ActualHandleTime} is null");
        if (!CenterToOrgTime.HasValue)
            throw new ArgumentNullException($"{CenterToOrgTime} is null");
        HandleDuration = (ActualHandleTime - CenterToOrgTime).Value.TotalMinutes;
    }

    public void SetAllDuration()
    {
        if (!CompleteTime.HasValue)
            throw new ArgumentNullException($"{CompleteTime} is null");
        AllDuration = (CompleteTime - CreationTime).Value.TotalMinutes;
    }

    #endregion
}

/// <summary>
/// 办理对象分组（以办理step分组，多人办理一个step为一组）
/// </summary>
public class HandlerGroupItem : IdName
{
    public string GroupId { get; set; }
}