﻿using XF.Domain.Dependency;
using XF.Domain.Exceptions;
using XF.Domain.Repository;

namespace Hotline.Settings;

public class SystemDomainService : ISystemDomainService, IScopeDependency
{
    private readonly ISystemOrganizeRepository _organizeRepository;
    private readonly IRepository<SystemDicData> _sysDicDataRepository;

    public SystemDomainService(
        ISystemOrganizeRepository organizeRepository,
        IRepository<SystemDicData> sysDicDataRepository)
    {
        _organizeRepository = organizeRepository;
        _sysDicDataRepository = sysDicDataRepository;
    }

    public async Task<IEnumerable<KeyValuePair<int, string>>> QueryOrgLevelOptionsAsync(CancellationToken cancellationToken)
    {
        var max = await _organizeRepository.Queryable()
            .MaxAsync(d => d.OrgLevel);

        var rsp = new List<KeyValuePair<int, string>>();
        for (int i = 1; i <= max; i++)
        {
            rsp.Add(new KeyValuePair<int, string>(i, $"{i}级部门"));
        }

        return rsp;
    }

    public async Task<IEnumerable<KeyValuePair<string, string>>> QueryOrgLevelStringOptionsAsync(CancellationToken cancellationToken)
    {
        var rsp = await QueryOrgLevelOptionsAsync(cancellationToken);
        return rsp.Select(d => new KeyValuePair<string, string>(d.Key.ToString(), d.Value));
    }

    public async Task<IReadOnlyList<SystemDicData>> GetSysDicDataByCodeAsync(string code, CancellationToken cancellationToken = default)
    {
        return await _sysDicDataRepository.Queryable().Where(x => x.DicTypeCode == code).ToTreeAsync(x => x.Children, x => x.ParentId, "");
    }

    /// <summary>
    /// 生成新的部门编码
    /// </summary>
    /// <param name="parentId"></param>
    /// <param name="cancellationToken"></param>
    /// <returns></returns>
    public async Task<string> GenerateNewOrgCodeAsync(string? parentId, CancellationToken cancellationToken)
    {
        if (string.IsNullOrEmpty(parentId))
        {
            //一级部门
            var maxCode = await _organizeRepository.Queryable()
                 .Where(d => d.OrgLevel == 1)
                 .MaxAsync(d => d.Id, cancellationToken);

            if (!int.TryParse(maxCode.GetLastOrgCode(), out var max))
                throw new UserFriendlyException("无效部门编码");
            return (max + 1).ToString("000");
        }
        else
        {
            var parentOrg = await _organizeRepository.GetAsync(parentId, cancellationToken);
            if (parentOrg == null)
                throw new UserFriendlyException("无效上级部门编码");

            var maxCode = await _organizeRepository.Queryable()
                .Where(d => d.ParentId == parentId)
                .MaxAsync(d => d.Id, cancellationToken);

            if (string.IsNullOrEmpty(maxCode))
                return $"{parentOrg.Id}001";

            if (!int.TryParse(maxCode.GetLastOrgCode(), out var max))
                throw new UserFriendlyException("无效部门编码");

            return $"{parentOrg.Id}{(max + 1):000}";
        }
    }

    /// <summary>
    /// 部门名称是否重复
    /// </summary>
    /// <param name="orgname"></param>
    /// <returns></returns>
    public async Task<int> IsNameRepeat(string orgname, CancellationToken cancellationToken)
    {
        return await _organizeRepository.CountAsync(x => x.OrgName == orgname, cancellationToken);
    }

    /// <summary>
    /// 部门简称是否重复
    /// </summary>
    /// <param name="orgShrotName"></param>
    /// <param name="cancellationToken"></param>
    /// <returns></returns>
    public async Task<int> IsShrotNameRepeat(string orgShrotName, CancellationToken cancellationToken)
    {
        return await _organizeRepository.CountAsync(x => x.OrgShortName == orgShrotName, cancellationToken);
    }
}